<template>
  <van-popup v-model="visible" position="bottom" :style="{ height: '100%' }" @open="onOpen">
    <van-nav-bar class="call-nav-bar" @click-left="close">
      <template #left>
        <van-icon name="cross" />
      </template>
      <template #title>
        <div class="title">
          <span class="name">参会人</span>
        </div>
      </template>
    </van-nav-bar>
    <van-search v-model="value" placeholder="搜索或呼叫" background="#1e263c" @input="onSearchChange" />
    <van-tabs v-if="!value" v-model="active">
      <!-- 调度中人员列表 -->
      <van-tab :title="tabsLabelConnecting">
        <div class="room-member-list">
          <!-- 调度员 -->
          <div class="room-member-list-item">
            <div class="info">
              <avatar :size="48">
                <div class="avatar">{{ currentUserProfile.nick }}</div>
              </avatar>
              <div class="name">
                <span class="value">{{ currentUserProfile.nick }}（我）</span>
                <span class="tag">调度员</span>
              </div>
            </div>
            <div class="stream-controller">
              <!-- 禁用音频轨道 -->
              <i v-if="isMutedAudio" class="btn btn-mute" @click="handleMuteAudio"></i>
              <!-- 启用音频轨道 -->
              <i v-if="!isMutedAudio" class="btn btn-unmute" @click="handleMuteAudio"></i>
              <!-- 禁用视频轨道 -->
              <i
                v-if="type === 'video' && yardmanType === '1' && isMutedVideo"
                class="btn btn-videoOn"
                @click="handleMuteVideo"
              ></i>
              <!-- 启用视频轨道 -->
              <i
                v-if="type === 'video' && yardmanType === '1' && !isMutedVideo"
                class="btn btn-videoOff"
                @click="handleMuteVideo"
              ></i>
            </div>
          </div>
          <!-- 被调度人 -->
          <div
            class="room-member-list-item"
            v-for="(item, index) in memberList.filter((x) => x.state !== 'DISCONNECTED')"
            :key="index"
          >
            <div class="info">
              <avatar :size="48">
                <div class="avatar">{{ item.nameSlice }}</div>
              </avatar>
              <div class="name">
                <span class="value">{{ item.name }}</span>
              </div>
            </div>
            <div class="stream-controller">
              <!-- 举手 -->
              <i class="btn btn-handup" v-show="item.action === 'handUp'"></i>
              <!-- 禁用音频轨道 -->
              <i
                v-if="RTCRemoteStreamList.find((x) => x.getUserId() === item.tencentUserId)?.hasAudio()"
                class="btn btn-unmute"
                @click="memberMute(item)"
              ></i>
              <!-- 启用音频轨道 -->
              <i
                v-if="!RTCRemoteStreamList.find((x) => x.getUserId() === item.tencentUserId)?.hasAudio()"
                class="btn btn-mute"
                @click="memberUnmute(item)"
              ></i>
              <!-- 禁用视频轨道 -->
              <i
                v-if="
                  type === 'video' &&
                  yardmanType === '1' &&
                  RTCRemoteStreamList.find((x) => x.getUserId() === item.tencentUserId)?.hasVideo()
                "
                class="btn btn-videoOff"
                @click="memberVideoOff(item)"
              ></i>
              <!-- 启用视频轨道 -->
              <i
                v-if="
                  type === 'video' &&
                  yardmanType === '1' &&
                  !RTCRemoteStreamList.find((x) => x.getUserId() === item.tencentUserId)?.hasVideo()
                "
                class="btn btn-videoOn"
                @click="memberVideoOn(item)"
              ></i>
              <!-- 移除成员 -->
              <i class="btn btn-kick-out" @click="kickOut(item.tencentUserId)"></i>
            </div>
          </div>
        </div>
      </van-tab>
      <!-- 挂断人员列表 -->
      <van-tab :title="tabsLabelDisconnecting">
        <div class="room-member-list">
          <div
            class="room-member-list-item"
            v-for="(item, index) in memberList.filter((x) => x.state === 'DISCONNECTED')"
            :key="index"
          >
            <div class="info">
              <avatar :size="48">
                <div class="avatar">{{ item.nameSlice }}</div>
              </avatar>
              <div class="name">
                <span class="value">{{ item.name }}</span>
              </div>
            </div>
            <div class="stream-controller">
              <span v-if="item.timeout" class="state-disconnected" style="color: #e2594e; font-size: 14px">未连上</span>
              <span v-if="item.beKickOut" class="state-disconnected" style="color: #e2594e; font-size: 14px">移除</span>
              <span v-if="!item.beKickOut && item.hangup" class="state-disconnected" style="color: #e2594e; font-size: 14px"
                >已挂断</span
              >
              <!-- 重拨 -->
              <i class="btn btn-recall" @click="recall(item)"></i>
            </div>
          </div>
        </div>
      </van-tab>
    </van-tabs>
    <!-- 可加入人员列表 -->
    <div v-if="value" class="room-member-list room-member-list-pick">
      <!-- 被调度人 -->
      <div class="room-member-list-item" v-for="(item, index) in pickupMemberList" :key="index">
        <div class="info">
          <avatar :size="48">
            <div class="avatar">{{ item.nameSlice }}</div>
          </avatar>
          <div class="name">
            <span class="value">{{ item.name }}</span>
          </div>
        </div>
        <div class="stream-controller">
          <!-- 呼叫 -->
          <van-button class="btn-member-call" plain hairline type="default" @click="addMember(item)">呼叫</van-button>
        </div>
      </div>
    </div>
    <div class="members-footer">
      <div class="members-footer-btn" @click="muteAll">全员静音</div>
      <div class="members-footer-btn" @click="unMuteAll">全员解除静音</div>
    </div>
  </van-popup>
</template>

<script>
import { NavBar, Popup, Icon, Tab, Tabs, Search, Button } from "vant";
import Avatar from "@/components/avatar.vue";
import rtc from "@/components/mixins/rtc.js";
import { mapState } from "vuex";
import { inviteUsers, kickOutUser, roomActions } from "@/api/room-controller";

export default {
  name: "calling-members",
  components: {
    [Popup.name]: Popup,
    [NavBar.name]: NavBar,
    [Icon.name]: Icon,
    [Tab.name]: Tab,
    [Tabs.name]: Tabs,
    [Search.name]: Search,
    [Button.name]: Button,
    Avatar,
  },
  mixins: [rtc],
  props: {
    visible: Boolean,
    // roomId: String,
    yardmanType: {
      // 调度类型：1视频调度 2 视频上拉：除了视频上拉传2，其他音频调度与视频调度都传1
      type: String,
      default: "1",
    },
    type: {
      type: String,
      default: "video",
      validate: (value) => {
        return ["video", "audio"].includes(value);
      },
    },
    memberList: {
      type: Array,
      default() {
        return [];
      },
    },
    allMembers: {
      type: Array,
      default() {
        return [];
      },
    },
  },
  data() {
    return {
      active: 0,
      value: "",
      pickupMemberList: [],
    };
  },
  computed: {
    ...mapState({
      currentUserProfile: (state) => state.user.currentUserProfile,
      roomId: (state) => state.trtc.roomId,
      RTCRemoteStreamList: (state) => state.trtc.remoteStreamList,
    }),
    tabsLabelConnecting() {
      return `调度中（${this.memberList.filter((x) => x.state !== "DISCONNECTED").length}）`;
    },
    tabsLabelDisconnecting() {
      return `断联（${this.memberList.filter((x) => x.state === "DISCONNECTED").length}）`;
    },
  },
  methods: {
    onOpen() {
      this.value = "";
      this.pickupMemberList = this.allMembers.filter((x) => !x.active);
    },
    close() {
      this.$emit("update:visible", false);
      this.$emit("close");
    },
    handleMuteAudio() {
      if (this.isMutedAudio) {
        this.unmuteAudio();
      } else {
        this.muteAudio();
      }
    },
    handleMuteVideo() {
      if (this.isMutedVideo) {
        this.unmuteVideo();
      } else {
        this.muteVideo();
      }
    },
    /**
     * @移除人员
     */
    kickOut(tencentUserId) {
      kickOutUser(this.roomId, tencentUserId)
        .then((res) => {
          console.log(res);
          // memberList移除踢出的成员
          const memberList = [...this.memberList];
          memberList.forEach((item) => {
            if (item.tencentUserId === tencentUserId) {
              item.state = "DISCONNECTED";
              item.beKickOut = true;
            }
          });
          this.$emit("update:memberList", memberList);
          this.$emit("kickOut", tencentUserId);
        })
        .catch(() => {
          //this.addFailedLog(`${err.message}`);
        });
    },
    recall(member) {
      inviteUsers(this.roomId, {
        roomUsers: [member].map((x) => {
          return {
            employeeId: x.id,
            organizationId: x.organizationId,
            tencentUserId: x.tencentUserId,
            securityStationId: x.securityStationId,
            superviseDepartId: x.superviseDepartId,
          };
        }),
      })
        .then(async (res) => {
          const { roomId, roomUsers } = res;
          console.log(roomUsers);
          console.log(roomId);
          // this.memberList.push(...members);
          roomUsers.forEach((user) => {
            this.allMembers.forEach((member) => {
              member.checked = false;
              if (member.id === user.employeeId) {
                member.active = true;
              }
            });
            this.memberList.forEach((member) => {
              if (member.id === user.employeeId) {
                member.state = "CONNECTING";
                member.beKickOut = false;
                member.hangup = false;
              }
            });
            this.$emit("update:memberList", JSON.parse(JSON.stringify(this.memberList)));
          });
          // 重新触发服务端拨号
          const memberList = roomUsers
            .map((x) => {
              return x.cid;
            })
            .join(",");
          const UseList = [member]
            .map((x) => {
              const { id, organizationId, tencentUserId, name } = x;
              // const item_cid = cid || tencentUserId
              // uidList.push(`${tencentUserId};${id};${organizationId};${name};${type}`)
              return `${tencentUserId};${id};${organizationId};${name};${this.type}`;
            })
            .join(",");
          // this.sendRoomMessage(roomId, memberList, UseList);
          this.$emit("sendVideoRoomMsg", roomId, memberList, UseList);
        })
        .catch(({ response: { data } }) => {
          console.log(data);
          const { message } = data;
          this.addFailedLog(message);
        });
    },
    /**
     * @全员禁音
     */
    muteAll() {
      // console.log("remoteStreamList",this.remoteStreamList);
      // this.remoteStreamList.forEach((remoteStream) => remoteStream.muteAudio());
      let tencentUserIds = [];
      roomActions(this.roomId, {
        action: "stopWord", //全员静音
        tencentUserIds: tencentUserIds.concat(
          this.memberList.map((x) => {
            return x.tencentUserId;
          })
        ),
      })
        .then(async () => {})
        .catch(() => {});
    },
    /**
     * @解除全员禁音
     */
    unMuteAll() {
      // this.remoteStreamList.forEach((remoteStream) => remoteStream.unmuteAudio());
      let tencentUserIds = [];
      roomActions(this.roomId, {
        action: "openWord", //全员解除静音
        tencentUserIds: tencentUserIds.concat(
          this.memberList.map((x) => {
            return x.tencentUserId;
          })
        ),
      })
        .then(async () => {})
        .catch(() => {});
    },
    /**
     * @参会人静音
     * @param member
     */
    memberMute({ tencentUserId }) {
      this.memberAction("stopWord", [tencentUserId]);
    },
    /**
     * @参会人解除静音
     * @param member
     */
    memberUnmute({ tencentUserId }) {
      console.log("memberUnmute", this.remoteStreamList);
      this.memberAction("openWord", [tencentUserId]);
    },
    /**
     * @参会人禁用视频
     * @param member
     */
    memberVideoOff({ tencentUserId }) {
      this.memberAction("camOff", [tencentUserId]);
    },
    /**
     * @参会人启用视频
     * @param member
     */
    memberVideoOn({ tencentUserId }) {
      this.memberAction("camOn", [tencentUserId]);
    },
    /**
     * @处理事件
     * @param action
     * @param tencentUserIds
     */
    memberAction(action, tencentUserIds) {
      roomActions(this.roomId, {
        action,
        tencentUserIds,
      })
        .then((res) => {
          console.log(res);
          // 如果发起动作为启、禁音视频。返回成功后去掉举手图标
          const memberList = [...this.memberList];
          tencentUserIds.forEach((id) => {
            memberList.forEach((member) => {
              if (member.tencentUserId === id) {
                member.action = "";
              }
            });
          });
          this.$emit("update:memberList", memberList);
        })
        .catch((err) => {
          console.log(err);
        });
    },
    onSearchChange(value) {
      console.log(value);
      console.log(this.allMembers);
      if (value) {
        this.pickupMemberList = this.allMembers.filter((x) => {
          return x.name.indexOf(value) >= 0 && !x.active && !x.checked;
        });
      } else {
        this.pickupMemberList = this.allMembers.filter((x) => !x.active && !x.checked);
      }
    },
    addMember(member) {
      console.log(member);
      inviteUsers(this.roomId, {
        roomUsers: [
          {
            employeeId: member.id,
            organizationId: member.organizationId,
            tencentUserId: member.tencentUserId,
            securityStationId: member.securityStationId,
            superviseDepartId: member.superviseDepartId,
          },
        ],
      })
        .then(async (res) => {
          const { roomId, roomUsers } = res;
          console.log(roomUsers);
          console.log(roomId);
          // this.memberList.push(...members);
          roomUsers.forEach((user) => {
            const member = this.allMembers.find((x) => x.id === user.employeeId);
            if (member) this.memberList.push(member);
            this.allMembers.forEach((member) => {
              member.checked = false;
              if (member.id === user.employeeId) {
                member.active = true;
              }
            });
          });
          // 重新触发服务端拨号
          const memberList = roomUsers
            .map((x) => {
              return x.cid;
            })
            .join(",");
          const UseList = [member]
            .map((x) => {
              const { id, organizationId, tencentUserId, name } = x;
              // const item_cid = cid || tencentUserId
              // uidList.push(`${tencentUserId};${id};${organizationId};${name};${type}`)
              return `${tencentUserId};${id};${organizationId};${name};${this.type}`;
            })
            .join(",");
          // this.sendRoomMessage(roomId, memberList, UseList);
          this.$emit("sendVideoRoomMsg", roomId, memberList, UseList);
        })
        .catch(() => {
          //this.addFailedLog(`${err.message}`);
        });
    },
  },
};
</script>

<style scoped lang="less">
@nav-bar-background-color: #1e263c;
@nav-bar-height: 58px;
@search-content-background-color: #000a27;
.van-hairline--bottom::after {
  border-bottom-width: 0;
}
.call-nav-bar {
  background: #1e263c;
  height: 58px;
  ::v-deep .van-nav-bar__content {
    height: 100%;
  }
  ::v-deep .van-icon {
    font-size: 30px;
    color: #ffffff;
  }
  .title {
    display: flex;
    flex-direction: column;
    align-items: center;
    .name {
      font-size: 14px;
      font-family: PingFang SC-Regular, PingFang SC, serif;
      font-weight: 400;
      color: #ffffff;
      line-height: 14px;
      margin-bottom: 8px;
    }
  }
}
::v-deep .van-search {
  height: 64px;
  .van-search__content {
    background-color: #000a27;
    height: 48px;
    .van-field__body {
      line-height: 38px;
      .van-field__control {
        color: #ffffff;
      }
    }
  }
}
::v-deep .van-tabs__wrap {
  height: 48px;
  .van-tab {
    background: #1b2237;
    font-size: 14px;
    font-family: PingFang SC-Regular, PingFang SC, serif;
    font-weight: 400;
    color: #ffffff;
    line-height: 17px;
    &--active {
      background: #32394b;
    }
  }
}
::v-deep .van-tabs__line {
  width: 50%;
  background-color: #0e70ff;
}
.room-member-list {
  width: 100%;
  height: calc(100vh - 58px - 64px - 48px - 85px);
  overflow-x: hidden;
  overflow-y: auto;
  padding: 16px 8px;
  background: #0e1425;
  &-pick {
    height: calc(100vh - 58px - 64px - 85px);
    box-sizing: border-box;
  }
  &-item {
    width: 100%;
    height: 66px;
    border-radius: 8px 8px 8px 8px;
    margin-bottom: 8px;
    display: flex;
    align-items: center;
    justify-content: space-between;
    padding: 0 16px;
    &:hover {
      background: #2b2b2b;
    }
    .info {
      display: flex;
      .avatar {
        width: 100%;
        height: 100%;
        border-radius: 50%;
        overflow: hidden;
        //margin-right: 16px;
        background: #0e71ff;
        font-size: 16px;
        font-weight: 400;
        color: #ffffff;
        //margin-right: 8px;
        display: flex;
        justify-content: center;
        align-items: center;
        img {
          width: 100%;
          height: auto;
        }
      }
      .name {
        display: flex;
        flex-direction: column;
        justify-content: center;
        align-items: flex-start;
        margin-left: 8px;
        .value {
          font-size: 12px;
          line-height: 20px;
          font-family: PingFang SC-Regular, PingFang SC, serif;
          font-weight: 400;
          color: #ffffff;
        }
        .tag {
          height: 20px;
          padding: 0 2px;
          background: #4c4c4c;
          border-radius: 4px 4px 4px 4px;
          font-size: 12px;
          font-family: PingFang SC-Regular, PingFang SC, serif;
          font-weight: 400;
          color: #ffffff;
          display: flex;
          align-items: center;
          justify-content: center;
        }
      }
    }
    .stream-controller {
      display: flex;
      align-items: center;
      .btn {
        width: 24px;
        height: 24px;
        cursor: pointer;
        margin-left: 16px;
        &-handup {
          background: url("~@/assets/images/conversation/举手.png") no-repeat;
          background-size: 100% 100%;
        }
        &-mute {
          background: url("~@/assets/images/conversation/静音1.png") no-repeat;
          background-size: 100% 100%;
        }
        &-unmute {
          background: url("~@/assets/images/conversation/开麦.png") no-repeat;
          background-size: 100% 100%;
        }
        &-videoOn {
          background: url("~@/assets/images/conversation/关闭摄像.png") no-repeat;
          background-size: 100% 100%;
        }
        &-videoOff {
          background: url("~@/assets/images/conversation/打开摄像.png") no-repeat;
          background-size: 100% 100%;
        }
        &-kick-out {
          background: url("~@/assets/images/conversation/移除.png") no-repeat;
          background-size: 100% 100%;
        }
        &-recall {
          background: url("~@/assets/images/conversation/拨通.png") no-repeat;
          background-size: 100% 100%;
        }
      }
      .btn-member-call {
        width: 80px;
        height: 40px;
        border-radius: 8px 8px 8px 8px;
        border: 1px solid #3b3b3b;
        background: transparent;
        font-size: 14px;
        font-family: PingFang SC-Regular, PingFang SC, serif;
        font-weight: 400;
        color: #ffffff;
        line-height: 20px;
      }
    }
  }
}
.members-footer {
  position: fixed;
  bottom: 0;
  left: 0;
  width: 100%;
  height: 85px;
  background: #1e263c;
  display: grid;
  grid-template-columns: 1fr 1fr;
  padding: 5px 0;
  &-btn {
    height: 48px;
    border-right: 1px solid #494e5f;
    padding: 0 13px;
    font-size: 14px;
    font-family: PingFang SC-Regular, PingFang SC, serif;
    font-weight: 400;
    color: #ffffff;
    display: flex;
    align-items: center;
    justify-content: center;
    &:last-child {
      border-right: 0;
    }
    &-active {
      background: #494e5f;
    }
  }
}
</style>
